@@ -162,7 +162,9 @@
       (make-jisx0208-to-0213-translation-pair ?$(O-y(B)
       (make-jisx0208-to-0213-translation-pair ?$(O-}(B)
       (make-jisx0208-to-0213-translation-pair ?$(O-~(B)
+      (make-jisx0208-to-0213-translation-pair ?$(O.!(B) ;; [2004]
       (make-jisx0208-to-0213-translation-pairs ?$(O."(B ?$(O/}(B)
+      (make-jisx0208-to-0213-translation-pair ?$(O/~(B) ;; [2004]
       (make-jisx0208-to-0213-translation-pair ?$(O0"(B)
       (make-jisx0208-to-0213-translation-pair ?$(O03(B)
       (make-jisx0208-to-0213-translation-pair ?$(O0o(B)
@@ -320,7 +322,9 @@
       (make-jisx0208-to-0213-translation-pair ?$(OO9(B)
       (make-jisx0208-to-0213-translation-pair ?$(OO:(B)
       (make-jisx0208-to-0213-translation-pair ?$(OO?(B)
+      (make-jisx0208-to-0213-translation-pair ?$(OOT(B) ;; [2004]
       (make-jisx0208-to-0213-translation-pairs ?$(OOU(B ?$(OO}(B)
+      (make-jisx0208-to-0213-translation-pair ?$(OO~(B) ;; [2004]
       (make-jisx0208-to-0213-translation-pair ?$(OPV(B)
       (make-jisx0208-to-0213-translation-pair ?$(OTd(B)
       (make-jisx0208-to-0213-translation-pair ?$(OYx(B)
@@ -347,7 +351,9 @@
       (make-jisx0208-to-0213-translation-pair ?$(OrM(B)
       (make-jisx0208-to-0213-translation-pair ?$(Ort(B)
       (make-jisx0208-to-0213-translation-pairs ?$(Ot!(B ?$(Ot&(B)
+      (make-jisx0208-to-0213-translation-pair ?$(Ot'(B) ;; [2004]
       (make-jisx0208-to-0213-translation-pairs ?$(Ot((B ?$(O~y(B)
+      (make-jisx0208-to-0213-translation-pairs ?$(O~z(B ?$(O~~(B) ;; [2004]
       ))))
 
 ;;;
